(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/animations'), require('rxjs'), require('rxjs/operators'), require('@angular/common'), require('@angular/common/http')) :
    typeof define === 'function' && define.amd ? define('ngx-block-loading', ['exports', '@angular/core', '@angular/animations', 'rxjs', 'rxjs/operators', '@angular/common', '@angular/common/http'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-block-loading'] = {}, global.ng.core, global.ng.animations, global.rxjs, global.rxjs.operators, global.ng.common, global.ng.common.http));
}(this, (function (exports, i0, i1, rxjs, operators, common, http) { 'use strict';

    var NGX_BLOCK_LOADING_OPTIONS = new i0.InjectionToken('NGX_BLOCK_LOADING_OPTIONS');
    var DEFAULT_OPTIONS = {
        routesToIgnore: [],
        inTime: '0.25s',
        outTime: '0.25s',
        loaderOutTime: '0.25s',
        containerHeight: '100px',
        loadingContainerClass: 'ngx-block-loading--container',
        loadingClass: 'ngx-block-loading',
        loadingFullPageClass: 'ngx-block-loading__full-page'
    };

    var AnimationHelperService = /** @class */ (function () {
        function AnimationHelperService(animationBuilder) {
            this.animationBuilder = animationBuilder;
        }
        AnimationHelperService.prototype.animate = function (hasAnimations, element, metadata, destroyOnDone, onDone) {
            var _this = this;
            if (destroyOnDone === void 0) { destroyOnDone = false; }
            if (element) {
                var notDonePlayers = hasAnimations.players.filter(function (player) { return !_this.isAnimationPlayerDone(player); });
                if (notDonePlayers.length > 0) {
                    var lastPlayerWrapper = notDonePlayers[notDonePlayers.length - 1];
                    lastPlayerWrapper.player.onDone(function () {
                        _this.runPlayer(hasAnimations, element, metadata, destroyOnDone, onDone);
                    });
                }
                else {
                    this.runPlayer(hasAnimations, element, metadata, destroyOnDone, onDone);
                }
            }
        };
        AnimationHelperService.prototype.runPlayer = function (hasAnimations, element, metadata, destroyOnDone, onDone) {
            var _this = this;
            if (destroyOnDone === void 0) { destroyOnDone = false; }
            var factory = this.animationBuilder.build(i1.useAnimation(metadata));
            var player = factory.create(element.nativeElement);
            var playerWrapper = {
                player: player,
                done: false
            };
            player.onDone(function () {
                playerWrapper.done = true;
                if (destroyOnDone) {
                    hasAnimations.players.forEach(function (animationPlayer) {
                        if (animationPlayer &&
                            !_this.isAnimationPlayerDone(animationPlayer)) {
                            animationPlayer.player.destroy();
                        }
                    });
                }
                if (hasAnimations.players.every(function (player) { return player.done; })) {
                    hasAnimations.players = [];
                }
                if (onDone) {
                    onDone();
                }
            });
            player.play();
            hasAnimations.players.push(playerWrapper);
        };
        AnimationHelperService.prototype.isAnimationPlayerDone = function (player) {
            return (player.done ||
                player.player.totalTime === player.player.getPosition());
        };
        return AnimationHelperService;
    }());
    AnimationHelperService.ɵprov = i0.ɵɵdefineInjectable({ factory: function AnimationHelperService_Factory() { return new AnimationHelperService(i0.ɵɵinject(i1.AnimationBuilder)); }, token: AnimationHelperService, providedIn: "root" });
    AnimationHelperService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    AnimationHelperService.ctorParameters = function () { return [
        { type: i1.AnimationBuilder }
    ]; };

    var NgxBlockLoadingService = /** @class */ (function () {
        function NgxBlockLoadingService() {
            this.loadingSource = new rxjs.BehaviorSubject(false);
            this.runningRequests = [];
            this.rendering = [];
        }
        Object.defineProperty(NgxBlockLoadingService.prototype, "loading", {
            get: function () {
                return this.loadingSource.value;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingService.prototype, "loadingAsync", {
            get: function () {
                return this.loadingSource;
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockLoadingService.prototype.addRunningRequest = function (request) {
            if (!this.runningRequests.includes(request)) {
                this.runningRequests.push(request);
            }
            this.changeLoading();
        };
        NgxBlockLoadingService.prototype.removeRunningRequest = function (request) {
            var requestIndex = this.runningRequests.indexOf(request);
            if (requestIndex > -1) {
                this.runningRequests.splice(requestIndex, 1);
            }
            this.changeLoading();
        };
        NgxBlockLoadingService.prototype.addRenderingElement = function (element) {
            var parent = element.nativeElement.parentElement;
            if (!this.rendering.some(function (e) { return e == parent; })) {
                this.rendering.push(parent);
                this.changeLoading();
            }
        };
        NgxBlockLoadingService.prototype.removeRenderingElement = function (element) {
            var parent = element.nativeElement.parentElement;
            var index = this.rendering.indexOf(parent);
            if (index !== -1) {
                this.rendering.splice(index, 1);
                this.changeLoading();
            }
        };
        NgxBlockLoadingService.prototype.changeLoading = function () {
            var val = this.runningRequests.length > 0 || this.rendering.length > 0;
            this.loadingSource.next(val);
        };
        return NgxBlockLoadingService;
    }());
    NgxBlockLoadingService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxBlockLoadingService_Factory() { return new NgxBlockLoadingService(); }, token: NgxBlockLoadingService, providedIn: "root" });
    NgxBlockLoadingService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];

    var NgxBlockLoadingDirective = /** @class */ (function () {
        function NgxBlockLoadingDirective(element, viewContainerRef, renderer, loadingService, options, animationHelper) {
            var _this = this;
            this.element = element;
            this.viewContainerRef = viewContainerRef;
            this.renderer = renderer;
            this.loadingService = loadingService;
            this.options = options;
            this.animationHelper = animationHelper;
            this._isLoading = false;
            this.hasLoadingElement = false;
            this.onDestroy$ = new rxjs.Subject();
            this.overrideLoading = false;
            this.players = [];
            this.inTime = this.options.inTime;
            this.outTime = this.options.outTime;
            this.loaderOutTime = this.options.loaderOutTime;
            this.containerHeight = this.options.containerHeight;
            this.loadingContainerClass = this.options.loadingContainerClass;
            this.loadingClass = this.options.loadingClass;
            this.loadingService.loadingSource
                .pipe(operators.takeUntil(this.onDestroy$))
                .subscribe(function (loading) {
                if (!_this.overrideLoading) {
                    _this.updateLoading(loading);
                }
            });
        }
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "isLoading", {
            get: function () {
                return this._isLoading;
            },
            set: function (value) {
                this._isLoading = value !== '' || value;
                // Only override loading if the value isn't blank
                // If the value is blank this means that the directive is empty
                this.overrideLoading = value !== '';
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "loadingContainerStyle", {
            get: function () {
                return i1.style({
                    height: this.containerHeight
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "loadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'visible',
                    opacity: 1
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "notLoadingContainerStyle", {
            get: function () {
                var _a, _b;
                return i1.style({
                    height: ((_b = (_a = this.element) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.scrollHeight) || i1.AUTO_STYLE
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingDirective.prototype, "notLoadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'hidden',
                    opacity: 0
                });
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockLoadingDirective.prototype.ngOnChanges = function (changes) {
            var isLoadingChanges = changes['isLoading'];
            if (isLoadingChanges) {
                this.updateLoadingElement(isLoadingChanges.currentValue);
            }
        };
        NgxBlockLoadingDirective.prototype.ngOnDestroy = function () {
            this.onDestroy$.next();
            this.onDestroy$.complete();
        };
        NgxBlockLoadingDirective.prototype.updateLoading = function (isLoading) {
            this.updateLoadingElement(isLoading);
        };
        NgxBlockLoadingDirective.prototype.updateLoadingElement = function (isLoading) {
            var _this = this;
            if (isLoading && !this.hasLoadingElement) {
                this.createLoadingElement();
            }
            else if (!isLoading && this.hasLoadingElement) {
                // Waiting a tick here because we need to let Angular finish rendering
                // It is likely that it actually hasn't
                setTimeout(function () { return _this.removeLoadingElement(); }, 0);
            }
        };
        NgxBlockLoadingDirective.prototype.getTemplateElement = function () {
            this.viewContainerRef.clear();
            var viewRef = this.viewContainerRef.createEmbeddedView(this.template);
            var element = viewRef.rootNodes[0];
            var duplicatedElement = element.cloneNode(true);
            this.viewContainerRef.clear();
            return new i0.ElementRef(duplicatedElement);
        };
        NgxBlockLoadingDirective.prototype.createLoadingElement = function () {
            this.renderer.addClass(this.element.nativeElement, this.loadingContainerClass);
            if (this.template) {
                this.loadingElement = this.getTemplateElement();
            }
            else {
                this.loadingElement = new i0.ElementRef(this.renderer.createElement('div'));
            }
            this.renderer.appendChild(this.element.nativeElement, this.loadingElement.nativeElement);
            this.renderer.addClass(this.loadingElement.nativeElement, this.loadingClass);
            this.hasLoadingElement = true;
            this.animationHelper.animate(this, this.element, i1.animation([
                this.notLoadingContainerStyle,
                i1.animate(this.inTime, this.loadingContainerStyle)
            ]));
        };
        NgxBlockLoadingDirective.prototype.removeLoadingElement = function () {
            var _this = this;
            this.players.forEach(function (player) {
                if (!_this.animationHelper.isAnimationPlayerDone(player)) {
                    player.player.destroy();
                }
            });
            this.animationHelper.animate(this, this.element, i1.animation([
                this.loadingContainerStyle,
                i1.animate(this.outTime, i1.style({
                    height: this.element.nativeElement.scrollHeight
                }))
            ]), true, function () {
                _this.hasLoadingElement = false;
                _this.animationHelper.animate(_this, _this.loadingElement, i1.animation([
                    _this.loadingStyle,
                    i1.animate(_this.loaderOutTime, _this.notLoadingStyle)
                ]), true, function () {
                    _this.renderer.removeChild(_this.element.nativeElement, _this.loadingElement.nativeElement);
                    _this.renderer.removeClass(_this.element.nativeElement, _this.loadingContainerClass);
                });
            });
        };
        return NgxBlockLoadingDirective;
    }());
    NgxBlockLoadingDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[ngxBlockLoading]'
                },] }
    ];
    NgxBlockLoadingDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: i0.ViewContainerRef },
        { type: i0.Renderer2 },
        { type: NgxBlockLoadingService },
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: AnimationHelperService }
    ]; };
    NgxBlockLoadingDirective.propDecorators = {
        inTime: [{ type: i0.Input }],
        outTime: [{ type: i0.Input }],
        loaderOutTime: [{ type: i0.Input }],
        containerHeight: [{ type: i0.Input }],
        loadingContainerClass: [{ type: i0.Input }],
        loadingClass: [{ type: i0.Input }],
        isLoading: [{ type: i0.Input, args: ['ngxBlockLoading',] }],
        template: [{ type: i0.Input }]
    };

    var NgxBlockRenderedDirective = /** @class */ (function () {
        function NgxBlockRenderedDirective(element, loadingService) {
            this.element = element;
            this.loadingService = loadingService;
            this._isRendered = false;
        }
        Object.defineProperty(NgxBlockRenderedDirective.prototype, "isRendered", {
            get: function () {
                return this._isRendered;
            },
            set: function (value) {
                this._isRendered = value !== '' || value;
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockRenderedDirective.prototype.ngOnChanges = function () {
            if (this.isRendered) {
                this.loadingService.removeRenderingElement(this.element);
            }
            else {
                this.loadingService.addRenderingElement(this.element);
            }
        };
        NgxBlockRenderedDirective.prototype.ngOnDestroy = function () {
            this.loadingService.removeRenderingElement(this.element);
        };
        return NgxBlockRenderedDirective;
    }());
    NgxBlockRenderedDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[ngxBlockRendered]'
                },] }
    ];
    NgxBlockRenderedDirective.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: NgxBlockLoadingService }
    ]; };
    NgxBlockRenderedDirective.propDecorators = {
        isRendered: [{ type: i0.Input, args: ['ngxBlockRendered',] }]
    };

    var NgxBlockLoadingInterceptor = /** @class */ (function () {
        function NgxBlockLoadingInterceptor(config, service) {
            this.service = service;
            this.routesToIgnore = config.routesToIgnore || [];
        }
        NgxBlockLoadingInterceptor.prototype.intercept = function (request, next) {
            var _this = this;
            if (!this.shouldIgnoreRoute(request)) {
                this.service.addRunningRequest(request.urlWithParams);
            }
            return next.handle(request).pipe(operators.finalize(function () {
                _this.service.removeRunningRequest(request.urlWithParams);
            }));
        };
        NgxBlockLoadingInterceptor.prototype.shouldIgnoreRoute = function (request) {
            var url = request.url;
            return (this.routesToIgnore.findIndex(function (route) { return typeof route === 'string'
                ? route === url
                : route instanceof RegExp
                    ? route.test(url)
                    : false; }) > -1);
        };
        return NgxBlockLoadingInterceptor;
    }());
    NgxBlockLoadingInterceptor.decorators = [
        { type: i0.Injectable }
    ];
    NgxBlockLoadingInterceptor.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: NgxBlockLoadingService }
    ]; };

    var loadingCount = 0;
    var isLoadingFullPage = new rxjs.BehaviorSubject(false);
    function updateLoading() {
        isLoadingFullPage.next(loadingCount > 0);
    }
    function fullPageLoading() {
        return function (source) {
            source
                .pipe(operators.finalize(function () {
                loadingCount--;
                updateLoading();
            }))
                .subscribe();
            loadingCount++;
            updateLoading();
            return source;
        };
    }

    var NgxBlockLoadingFullPageComponent = /** @class */ (function () {
        function NgxBlockLoadingFullPageComponent(options, animationHelper) {
            var _this = this;
            this.options = options;
            this.animationHelper = animationHelper;
            this.loading = false;
            this.players = [];
            this.onDestroy$ = new rxjs.Subject();
            this.loaderOutTime = this.options.loaderOutTime;
            this.loadingClass = this.options.loadingClass;
            this.loadingFullPageClass = this.options.loadingFullPageClass;
            isLoadingFullPage
                .pipe(operators.takeUntil(this.onDestroy$))
                .subscribe(function (loading) {
                if (_this.isLoading == undefined) {
                    _this.updateLoading(loading);
                }
            });
        }
        Object.defineProperty(NgxBlockLoadingFullPageComponent.prototype, "loadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'visible',
                    opacity: 1
                });
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(NgxBlockLoadingFullPageComponent.prototype, "notLoadingStyle", {
            get: function () {
                return i1.style({
                    visibility: 'hidden',
                    opacity: 0
                });
            },
            enumerable: false,
            configurable: true
        });
        NgxBlockLoadingFullPageComponent.prototype.ngOnChanges = function (changes) {
            var loadingChanges = changes['loading'];
            if (loadingChanges) {
                this.updateLoading(loadingChanges.currentValue);
            }
        };
        NgxBlockLoadingFullPageComponent.prototype.updateLoading = function (loading) {
            if (loading) {
                this.loading = true;
            }
            else {
                this.removeFullPageLoading();
            }
        };
        NgxBlockLoadingFullPageComponent.prototype.removeFullPageLoading = function () {
            var _this = this;
            this.animationHelper.animate(this, this.element, i1.animation([
                this.loadingStyle,
                i1.animate(this.loaderOutTime, this.notLoadingStyle)
            ]), true, function () {
                _this.loading = false;
            });
        };
        return NgxBlockLoadingFullPageComponent;
    }());
    NgxBlockLoadingFullPageComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'ngx-block-loading-full-page',
                    template: "<div #element>\r\n  <ng-container *ngIf=\"loading && template\">\r\n    <ng-container *ngTemplateOutlet=\"template\"></ng-container>\r\n  </ng-container>\r\n\r\n  <div class=\"{{loadingClass}} {{loadingFullPageClass}}\" *ngIf=\"loading && !template\"></div>\r\n</div>\r\n",
                    styles: [""]
                },] }
    ];
    NgxBlockLoadingFullPageComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: i0.Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
        { type: AnimationHelperService }
    ]; };
    NgxBlockLoadingFullPageComponent.propDecorators = {
        template: [{ type: i0.Input }],
        loaderOutTime: [{ type: i0.Input }],
        loadingClass: [{ type: i0.Input }],
        loadingFullPageClass: [{ type: i0.Input }],
        isLoading: [{ type: i0.Input, args: ['loading',] }],
        element: [{ type: i0.ViewChild, args: ['element', { static: false },] }]
    };

    var NgxBlockLoadingModule = /** @class */ (function () {
        function NgxBlockLoadingModule() {
        }
        NgxBlockLoadingModule.forRoot = function (options) {
            return {
                ngModule: NgxBlockLoadingModule,
                providers: [
                    {
                        provide: http.HTTP_INTERCEPTORS,
                        useClass: NgxBlockLoadingInterceptor,
                        multi: true
                    },
                    (options === null || options === void 0 ? void 0 : options.provider) || {
                        provide: NGX_BLOCK_LOADING_OPTIONS,
                        useValue: setDefaultOptions(options === null || options === void 0 ? void 0 : options.config)
                    }
                ]
            };
        };
        return NgxBlockLoadingModule;
    }());
    NgxBlockLoadingModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [common.CommonModule],
                    declarations: [
                        NgxBlockLoadingDirective,
                        NgxBlockRenderedDirective,
                        NgxBlockLoadingFullPageComponent
                    ],
                    exports: [
                        NgxBlockLoadingDirective,
                        NgxBlockRenderedDirective,
                        NgxBlockLoadingFullPageComponent
                    ]
                },] }
    ];
    function setDefaultOptions(options) {
        var defaultOptions = DEFAULT_OPTIONS;
        return {
            routesToIgnore: (options === null || options === void 0 ? void 0 : options.routesToIgnore) || defaultOptions.routesToIgnore,
            loadingTemplate: (options === null || options === void 0 ? void 0 : options.loadingTemplate) || defaultOptions.loadingTemplate,
            inTime: (options === null || options === void 0 ? void 0 : options.inTime) || defaultOptions.inTime,
            outTime: (options === null || options === void 0 ? void 0 : options.outTime) || defaultOptions.outTime,
            loaderOutTime: (options === null || options === void 0 ? void 0 : options.loaderOutTime) || defaultOptions.loaderOutTime,
            containerHeight: (options === null || options === void 0 ? void 0 : options.containerHeight) || defaultOptions.containerHeight,
            loadingContainerClass: (options === null || options === void 0 ? void 0 : options.loadingContainerClass) ||
                defaultOptions.loadingContainerClass,
            loadingClass: (options === null || options === void 0 ? void 0 : options.loadingClass) || defaultOptions.loadingClass,
            loadingFullPageClass: (options === null || options === void 0 ? void 0 : options.loadingFullPageClass) || defaultOptions.loadingFullPageClass
        };
    }

    /*
     * Public API Surface of loading
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NGX_BLOCK_LOADING_OPTIONS = NGX_BLOCK_LOADING_OPTIONS;
    exports.NgxBlockLoadingDirective = NgxBlockLoadingDirective;
    exports.NgxBlockLoadingFullPageComponent = NgxBlockLoadingFullPageComponent;
    exports.NgxBlockLoadingInterceptor = NgxBlockLoadingInterceptor;
    exports.NgxBlockLoadingModule = NgxBlockLoadingModule;
    exports.NgxBlockRenderedDirective = NgxBlockRenderedDirective;
    exports.fullPageLoading = fullPageLoading;
    exports.ɵa = NgxBlockLoadingService;
    exports.ɵb = AnimationHelperService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-block-loading.umd.js.map
