import { InjectionToken, ɵɵdefineInjectable, ɵɵinject, Injectable, ElementRef, Directive, ViewContainerRef, Renderer2, Inject, Input, Component, ViewChild, NgModule } from '@angular/core';
import { useAnimation, AnimationBuilder, style, AUTO_STYLE, animation, animate } from '@angular/animations';
import { BehaviorSubject, Subject } from 'rxjs';
import { takeUntil, finalize } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { HTTP_INTERCEPTORS } from '@angular/common/http';

const NGX_BLOCK_LOADING_OPTIONS = new InjectionToken('NGX_BLOCK_LOADING_OPTIONS');
const DEFAULT_OPTIONS = {
    routesToIgnore: [],
    inTime: '0.25s',
    outTime: '0.25s',
    loaderOutTime: '0.25s',
    containerHeight: '100px',
    loadingContainerClass: 'ngx-block-loading--container',
    loadingClass: 'ngx-block-loading',
    loadingFullPageClass: 'ngx-block-loading__full-page'
};

class AnimationHelperService {
    constructor(animationBuilder) {
        this.animationBuilder = animationBuilder;
    }
    animate(hasAnimations, element, metadata, destroyOnDone = false, onDone) {
        if (element) {
            const notDonePlayers = hasAnimations.players.filter(player => !this.isAnimationPlayerDone(player));
            if (notDonePlayers.length > 0) {
                const lastPlayerWrapper = notDonePlayers[notDonePlayers.length - 1];
                lastPlayerWrapper.player.onDone(() => {
                    this.runPlayer(hasAnimations, element, metadata, destroyOnDone, onDone);
                });
            }
            else {
                this.runPlayer(hasAnimations, element, metadata, destroyOnDone, onDone);
            }
        }
    }
    runPlayer(hasAnimations, element, metadata, destroyOnDone = false, onDone) {
        const factory = this.animationBuilder.build(useAnimation(metadata));
        const player = factory.create(element.nativeElement);
        const playerWrapper = {
            player: player,
            done: false
        };
        player.onDone(() => {
            playerWrapper.done = true;
            if (destroyOnDone) {
                hasAnimations.players.forEach(animationPlayer => {
                    if (animationPlayer &&
                        !this.isAnimationPlayerDone(animationPlayer)) {
                        animationPlayer.player.destroy();
                    }
                });
            }
            if (hasAnimations.players.every(player => player.done)) {
                hasAnimations.players = [];
            }
            if (onDone) {
                onDone();
            }
        });
        player.play();
        hasAnimations.players.push(playerWrapper);
    }
    isAnimationPlayerDone(player) {
        return (player.done ||
            player.player.totalTime === player.player.getPosition());
    }
}
AnimationHelperService.ɵprov = ɵɵdefineInjectable({ factory: function AnimationHelperService_Factory() { return new AnimationHelperService(ɵɵinject(AnimationBuilder)); }, token: AnimationHelperService, providedIn: "root" });
AnimationHelperService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
AnimationHelperService.ctorParameters = () => [
    { type: AnimationBuilder }
];

class NgxBlockLoadingService {
    constructor() {
        this.loadingSource = new BehaviorSubject(false);
        this.runningRequests = [];
        this.rendering = [];
    }
    get loading() {
        return this.loadingSource.value;
    }
    get loadingAsync() {
        return this.loadingSource;
    }
    addRunningRequest(request) {
        if (!this.runningRequests.includes(request)) {
            this.runningRequests.push(request);
        }
        this.changeLoading();
    }
    removeRunningRequest(request) {
        const requestIndex = this.runningRequests.indexOf(request);
        if (requestIndex > -1) {
            this.runningRequests.splice(requestIndex, 1);
        }
        this.changeLoading();
    }
    addRenderingElement(element) {
        const parent = element.nativeElement.parentElement;
        if (!this.rendering.some(e => e == parent)) {
            this.rendering.push(parent);
            this.changeLoading();
        }
    }
    removeRenderingElement(element) {
        const parent = element.nativeElement.parentElement;
        const index = this.rendering.indexOf(parent);
        if (index !== -1) {
            this.rendering.splice(index, 1);
            this.changeLoading();
        }
    }
    changeLoading() {
        const val = this.runningRequests.length > 0 || this.rendering.length > 0;
        this.loadingSource.next(val);
    }
}
NgxBlockLoadingService.ɵprov = ɵɵdefineInjectable({ factory: function NgxBlockLoadingService_Factory() { return new NgxBlockLoadingService(); }, token: NgxBlockLoadingService, providedIn: "root" });
NgxBlockLoadingService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

class NgxBlockLoadingDirective {
    constructor(element, viewContainerRef, renderer, loadingService, options, animationHelper) {
        this.element = element;
        this.viewContainerRef = viewContainerRef;
        this.renderer = renderer;
        this.loadingService = loadingService;
        this.options = options;
        this.animationHelper = animationHelper;
        this._isLoading = false;
        this.hasLoadingElement = false;
        this.onDestroy$ = new Subject();
        this.overrideLoading = false;
        this.players = [];
        this.inTime = this.options.inTime;
        this.outTime = this.options.outTime;
        this.loaderOutTime = this.options.loaderOutTime;
        this.containerHeight = this.options.containerHeight;
        this.loadingContainerClass = this.options.loadingContainerClass;
        this.loadingClass = this.options.loadingClass;
        this.loadingService.loadingSource
            .pipe(takeUntil(this.onDestroy$))
            .subscribe(loading => {
            if (!this.overrideLoading) {
                this.updateLoading(loading);
            }
        });
    }
    set isLoading(value) {
        this._isLoading = value !== '' || value;
        // Only override loading if the value isn't blank
        // If the value is blank this means that the directive is empty
        this.overrideLoading = value !== '';
    }
    get isLoading() {
        return this._isLoading;
    }
    get loadingContainerStyle() {
        return style({
            height: this.containerHeight
        });
    }
    get loadingStyle() {
        return style({
            visibility: 'visible',
            opacity: 1
        });
    }
    get notLoadingContainerStyle() {
        var _a, _b;
        return style({
            height: ((_b = (_a = this.element) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.scrollHeight) || AUTO_STYLE
        });
    }
    get notLoadingStyle() {
        return style({
            visibility: 'hidden',
            opacity: 0
        });
    }
    ngOnChanges(changes) {
        const isLoadingChanges = changes['isLoading'];
        if (isLoadingChanges) {
            this.updateLoadingElement(isLoadingChanges.currentValue);
        }
    }
    ngOnDestroy() {
        this.onDestroy$.next();
        this.onDestroy$.complete();
    }
    updateLoading(isLoading) {
        this.updateLoadingElement(isLoading);
    }
    updateLoadingElement(isLoading) {
        if (isLoading && !this.hasLoadingElement) {
            this.createLoadingElement();
        }
        else if (!isLoading && this.hasLoadingElement) {
            // Waiting a tick here because we need to let Angular finish rendering
            // It is likely that it actually hasn't
            setTimeout(() => this.removeLoadingElement(), 0);
        }
    }
    getTemplateElement() {
        this.viewContainerRef.clear();
        const viewRef = this.viewContainerRef.createEmbeddedView(this.template);
        const element = viewRef.rootNodes[0];
        const duplicatedElement = element.cloneNode(true);
        this.viewContainerRef.clear();
        return new ElementRef(duplicatedElement);
    }
    createLoadingElement() {
        this.renderer.addClass(this.element.nativeElement, this.loadingContainerClass);
        if (this.template) {
            this.loadingElement = this.getTemplateElement();
        }
        else {
            this.loadingElement = new ElementRef(this.renderer.createElement('div'));
        }
        this.renderer.appendChild(this.element.nativeElement, this.loadingElement.nativeElement);
        this.renderer.addClass(this.loadingElement.nativeElement, this.loadingClass);
        this.hasLoadingElement = true;
        this.animationHelper.animate(this, this.element, animation([
            this.notLoadingContainerStyle,
            animate(this.inTime, this.loadingContainerStyle)
        ]));
    }
    removeLoadingElement() {
        this.players.forEach(player => {
            if (!this.animationHelper.isAnimationPlayerDone(player)) {
                player.player.destroy();
            }
        });
        this.animationHelper.animate(this, this.element, animation([
            this.loadingContainerStyle,
            animate(this.outTime, style({
                height: this.element.nativeElement.scrollHeight
            }))
        ]), true, () => {
            this.hasLoadingElement = false;
            this.animationHelper.animate(this, this.loadingElement, animation([
                this.loadingStyle,
                animate(this.loaderOutTime, this.notLoadingStyle)
            ]), true, () => {
                this.renderer.removeChild(this.element.nativeElement, this.loadingElement.nativeElement);
                this.renderer.removeClass(this.element.nativeElement, this.loadingContainerClass);
            });
        });
    }
}
NgxBlockLoadingDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxBlockLoading]'
            },] }
];
NgxBlockLoadingDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: ViewContainerRef },
    { type: Renderer2 },
    { type: NgxBlockLoadingService },
    { type: undefined, decorators: [{ type: Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
    { type: AnimationHelperService }
];
NgxBlockLoadingDirective.propDecorators = {
    inTime: [{ type: Input }],
    outTime: [{ type: Input }],
    loaderOutTime: [{ type: Input }],
    containerHeight: [{ type: Input }],
    loadingContainerClass: [{ type: Input }],
    loadingClass: [{ type: Input }],
    isLoading: [{ type: Input, args: ['ngxBlockLoading',] }],
    template: [{ type: Input }]
};

class NgxBlockRenderedDirective {
    constructor(element, loadingService) {
        this.element = element;
        this.loadingService = loadingService;
        this._isRendered = false;
    }
    set isRendered(value) {
        this._isRendered = value !== '' || value;
    }
    get isRendered() {
        return this._isRendered;
    }
    ngOnChanges() {
        if (this.isRendered) {
            this.loadingService.removeRenderingElement(this.element);
        }
        else {
            this.loadingService.addRenderingElement(this.element);
        }
    }
    ngOnDestroy() {
        this.loadingService.removeRenderingElement(this.element);
    }
}
NgxBlockRenderedDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngxBlockRendered]'
            },] }
];
NgxBlockRenderedDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgxBlockLoadingService }
];
NgxBlockRenderedDirective.propDecorators = {
    isRendered: [{ type: Input, args: ['ngxBlockRendered',] }]
};

class NgxBlockLoadingInterceptor {
    constructor(config, service) {
        this.service = service;
        this.routesToIgnore = config.routesToIgnore || [];
    }
    intercept(request, next) {
        if (!this.shouldIgnoreRoute(request)) {
            this.service.addRunningRequest(request.urlWithParams);
        }
        return next.handle(request).pipe(finalize(() => {
            this.service.removeRunningRequest(request.urlWithParams);
        }));
    }
    shouldIgnoreRoute(request) {
        const url = request.url;
        return (this.routesToIgnore.findIndex(route => typeof route === 'string'
            ? route === url
            : route instanceof RegExp
                ? route.test(url)
                : false) > -1);
    }
}
NgxBlockLoadingInterceptor.decorators = [
    { type: Injectable }
];
NgxBlockLoadingInterceptor.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
    { type: NgxBlockLoadingService }
];

let loadingCount = 0;
const isLoadingFullPage = new BehaviorSubject(false);
function updateLoading() {
    isLoadingFullPage.next(loadingCount > 0);
}
function fullPageLoading() {
    return function (source) {
        source
            .pipe(finalize(() => {
            loadingCount--;
            updateLoading();
        }))
            .subscribe();
        loadingCount++;
        updateLoading();
        return source;
    };
}

class NgxBlockLoadingFullPageComponent {
    constructor(options, animationHelper) {
        this.options = options;
        this.animationHelper = animationHelper;
        this.loading = false;
        this.players = [];
        this.onDestroy$ = new Subject();
        this.loaderOutTime = this.options.loaderOutTime;
        this.loadingClass = this.options.loadingClass;
        this.loadingFullPageClass = this.options.loadingFullPageClass;
        isLoadingFullPage
            .pipe(takeUntil(this.onDestroy$))
            .subscribe(loading => {
            if (this.isLoading == undefined) {
                this.updateLoading(loading);
            }
        });
    }
    get loadingStyle() {
        return style({
            visibility: 'visible',
            opacity: 1
        });
    }
    get notLoadingStyle() {
        return style({
            visibility: 'hidden',
            opacity: 0
        });
    }
    ngOnChanges(changes) {
        const loadingChanges = changes['loading'];
        if (loadingChanges) {
            this.updateLoading(loadingChanges.currentValue);
        }
    }
    updateLoading(loading) {
        if (loading) {
            this.loading = true;
        }
        else {
            this.removeFullPageLoading();
        }
    }
    removeFullPageLoading() {
        this.animationHelper.animate(this, this.element, animation([
            this.loadingStyle,
            animate(this.loaderOutTime, this.notLoadingStyle)
        ]), true, () => {
            this.loading = false;
        });
    }
}
NgxBlockLoadingFullPageComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-block-loading-full-page',
                template: "<div #element>\r\n  <ng-container *ngIf=\"loading && template\">\r\n    <ng-container *ngTemplateOutlet=\"template\"></ng-container>\r\n  </ng-container>\r\n\r\n  <div class=\"{{loadingClass}} {{loadingFullPageClass}}\" *ngIf=\"loading && !template\"></div>\r\n</div>\r\n",
                styles: [""]
            },] }
];
NgxBlockLoadingFullPageComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [NGX_BLOCK_LOADING_OPTIONS,] }] },
    { type: AnimationHelperService }
];
NgxBlockLoadingFullPageComponent.propDecorators = {
    template: [{ type: Input }],
    loaderOutTime: [{ type: Input }],
    loadingClass: [{ type: Input }],
    loadingFullPageClass: [{ type: Input }],
    isLoading: [{ type: Input, args: ['loading',] }],
    element: [{ type: ViewChild, args: ['element', { static: false },] }]
};

class NgxBlockLoadingModule {
    static forRoot(options) {
        return {
            ngModule: NgxBlockLoadingModule,
            providers: [
                {
                    provide: HTTP_INTERCEPTORS,
                    useClass: NgxBlockLoadingInterceptor,
                    multi: true
                },
                (options === null || options === void 0 ? void 0 : options.provider) || {
                    provide: NGX_BLOCK_LOADING_OPTIONS,
                    useValue: setDefaultOptions(options === null || options === void 0 ? void 0 : options.config)
                }
            ]
        };
    }
}
NgxBlockLoadingModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [
                    NgxBlockLoadingDirective,
                    NgxBlockRenderedDirective,
                    NgxBlockLoadingFullPageComponent
                ],
                exports: [
                    NgxBlockLoadingDirective,
                    NgxBlockRenderedDirective,
                    NgxBlockLoadingFullPageComponent
                ]
            },] }
];
function setDefaultOptions(options) {
    const defaultOptions = DEFAULT_OPTIONS;
    return {
        routesToIgnore: (options === null || options === void 0 ? void 0 : options.routesToIgnore) || defaultOptions.routesToIgnore,
        loadingTemplate: (options === null || options === void 0 ? void 0 : options.loadingTemplate) || defaultOptions.loadingTemplate,
        inTime: (options === null || options === void 0 ? void 0 : options.inTime) || defaultOptions.inTime,
        outTime: (options === null || options === void 0 ? void 0 : options.outTime) || defaultOptions.outTime,
        loaderOutTime: (options === null || options === void 0 ? void 0 : options.loaderOutTime) || defaultOptions.loaderOutTime,
        containerHeight: (options === null || options === void 0 ? void 0 : options.containerHeight) || defaultOptions.containerHeight,
        loadingContainerClass: (options === null || options === void 0 ? void 0 : options.loadingContainerClass) ||
            defaultOptions.loadingContainerClass,
        loadingClass: (options === null || options === void 0 ? void 0 : options.loadingClass) || defaultOptions.loadingClass,
        loadingFullPageClass: (options === null || options === void 0 ? void 0 : options.loadingFullPageClass) || defaultOptions.loadingFullPageClass
    };
}

/*
 * Public API Surface of loading
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGX_BLOCK_LOADING_OPTIONS, NgxBlockLoadingDirective, NgxBlockLoadingFullPageComponent, NgxBlockLoadingInterceptor, NgxBlockLoadingModule, NgxBlockRenderedDirective, fullPageLoading, NgxBlockLoadingService as ɵa, AnimationHelperService as ɵb };
//# sourceMappingURL=ngx-block-loading.js.map
